package xtime

import (
	"database/sql/driver"
	"fmt"
	"regexp"
	"strconv"
	"time"
)

const (
	Layout_YYYYMMMDDHHMMSS = "20060102150405"
	Layout_DateTimeWithMS  = "2006-01-02 15:04:05.000"
	Layout_DateTime        = "2006-01-02 15:04:05"
	Layout_Date            = "2006-01-02"
	Layout_Time            = "15:04:05"
)

// 定义time.Time的别名
type DateTime time.Time

// 自定义json序列化
func (x DateTime) MarshalJSON() ([]byte, error) {
	if x.IsZero() {
		return []byte(`""`), nil
	}
	var jsonTimeStr = fmt.Sprintf(`"%s"`, time.Time(x).Format("2006-01-02 15:04:05"))
	return []byte(jsonTimeStr), nil
}

// 自定义json反序列化
func (x *DateTime) UnmarshalJSON(data []byte) error {
	if len(data) == 0 || string(data) == `""` {
		*x = DateTime{}
		return nil
	}
	t, err := time.Parse(`"2006-01-02 15:04:05"`, string(data))
	if err != nil {
		return err
	}
	*x = DateTime(t)
	return nil
}

func (x DateTime) Time() time.Time {
	return time.Time(x)
}

func (x DateTime) IsZero() bool {
	return x.Time().IsZero()
}
func (x DateTime) IsNotZero() bool {
	return !x.IsZero()
}

func NewDateTime(t time.Time) DateTime {
	return DateTime(t)
}

func Now() DateTime {
	return DateTime(time.Now())
}

func NowPtr() *DateTime {
	j := DateTime(time.Now())
	return &j
}

const TimeFormat = "2006-01-02 15:04:05"

// Value insert timestamp into mysql need this function.
func (t DateTime) Value() (driver.Value, error) {
	var zeroTime time.Time
	if time.Time(t).UnixNano() == zeroTime.UnixNano() {
		return nil, nil
	}
	return time.Time(t), nil
}

// Scan value of time.Time
func (t *DateTime) Scan(v interface{}) error {
	value, ok := v.(time.Time)
	if ok {
		*t = DateTime(value)
		return nil
	}
	return fmt.Errorf("can not convert %v to timestamp", v)
}

func BeginOfDay(t time.Time) DateTime {
	y, m, d := t.Date()
	begin := time.Date(y, m, d, 0, 0, 0, 0, t.Location())
	return DateTime(begin)
}

func EndOfDay(t time.Time) DateTime {
	y, m, d := t.Date()
	end := time.Date(y, m, d, 23, 59, 59, int(time.Second-time.Nanosecond), t.Location())
	return DateTime(end)
}

func TodayBegin() DateTime {
	t := time.Now()
	y, m, d := t.Date()
	begin := time.Date(y, m, d, 0, 0, 0, 0, t.Location())
	return DateTime(begin)
}

func TodayEnd() DateTime {
	t := time.Now()
	y, m, d := t.Date()
	end := time.Date(y, m, d, 23, 59, 59, int(time.Second-time.Nanosecond), t.Location())
	return DateTime(end)
}

var durationRegex = regexp.MustCompile(`(\d+)([a-zA-Z]+)`)

func ParseExtendedDuration(s string) (time.Duration, error) {
	matches := durationRegex.FindAllStringSubmatch(s, -1)
	if len(matches) == 0 {
		return 0, fmt.Errorf("invalid format: %q", s)
	}

	// 验证完整匹配
	var matched string
	for _, m := range matches {
		matched += m[0]
	}
	if matched != s {
		return 0, fmt.Errorf("invalid characters in duration: %q", s)
	}

	var total time.Duration
	for _, match := range matches {
		numStr := match[1]
		unit := match[2]
		// unit := strings.ToUpper(match[2])
		num, err := strconv.ParseInt(numStr, 10, 64)
		if err != nil {
			return 0, fmt.Errorf("invalid number %q: %v", numStr, err)
		}

		var d time.Duration
		switch unit {
		case "d", "D":
			d = time.Duration(num) * 24 * time.Hour
		case "w", "W":
			d = time.Duration(num) * 7 * 24 * time.Hour
		case "M":
			d = time.Duration(num) * 30 * 24 * time.Hour
		case "y", "Y":
			d = time.Duration(num) * 365 * 24 * time.Hour
		default:
			// 回退到标准库解析
			if dur, err := time.ParseDuration(match[0]); err == nil {
				d = dur
			} else {
				return 0, fmt.Errorf("unknown unit %q", unit)
			}
		}
		total += d
	}

	return total, nil
}
